/*! babel-plugin-smart-webpack-import v1.0.3 by Sebastian Software <s.werner@sebastian-software.de> */
'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var json5 = _interopDefault(require('json5'));
var path = require('path');
var crypto = _interopDefault(require('crypto'));
var basex = _interopDefault(require('base-x'));

/* eslint-disable filenames/match-exported */
var base62 = basex("0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ");
var DEFAULT_LENGTH = 5;

function hashString(input) {
  var precision = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : DEFAULT_LENGTH;
  return base62.encode(crypto.createHash("sha256").update(input).digest()).slice(0, precision);
}

function getImportArgPath(path$$1) {
  return path$$1.parentPath.get("arguments")[0];
}

function smartWebpackImport(_ref) {
  var types = _ref.types,
      template = _ref.template;
  var visited = Symbol("visited");
  return {
    name: "smart-webpack-import",
    visitor: {
      Import: function Import(path$$1, state) {
        /* eslint-disable immutable/no-mutation */
        if (path$$1[visited]) {
          return;
        }

        path$$1[visited] = true;
        var importArg = getImportArgPath(path$$1);
        var importArgNode = importArg.node;
        var quasis = importArgNode.quasis,
            expressions = importArgNode.expressions,
            leadingComments = importArgNode.leadingComments;
        var requester = path.dirname(state.file.opts.filename);
        var request = quasis ? quasis[0].value.cooked : importArgNode.value; // There exists the possibility of non usable value. Typically only
        // when the user has import() statements with other complex data, but
        // not a string or template string. We handle this gracefully by ignoring.

        if (request == null) {
          return;
        }

        var jsonContent = {}; // Try to parse all previous comments

        if (leadingComments) {
          leadingComments.forEach(function (comment, index) {
            // Skip empty comments
            if (!comment.value.trim()) {
              return;
            } // Webpack magic comments are declared as JSON5 but miss the curly braces.


            var parsed;

            try {
              parsed = json5.parse("{".concat(comment.value, "}"));
            } catch (err) {
              // Most probably a non JSON5 comment
              return;
            } // Skip comment processing if it already contains a chunk name


            if (parsed.webpackChunkName) {
              jsonContent.webpackChunkName = true;
              return;
            } // We copy over all fields and...


            for (var key in parsed) {
              jsonContent[key] = parsed[key];
            } // Cleanup the parsed comment afterwards


            comment.value = "";
          });
        }

        if (!jsonContent.webpackChunkName) {
          var hasExpressions = expressions && expressions.length > 0; // Append [request] as placeholder for dynamic part in WebpackChunkName

          var fullRequest = hasExpressions ? "".concat(request, "[request]") : request; // Prepend some clean identifier of the static part when using expressions.
          // This is not required to work, but helps users to identify different chunks.

          var requestPrefix = hasExpressions ? request.replace(/^[./]+|(\.js$)/g, "").replace(/\//g, "-") : ""; // Cleanup combined request to not contain any paths info

          var plainRequest = path.basename(fullRequest, path.extname(fullRequest)); // Hash request origin and request

          var importHash = hashString("".concat(requester, "::").concat(request)); // Add our chunk name to the previously parsed values

          jsonContent.webpackChunkName = "".concat(requestPrefix).concat(plainRequest, "-").concat(importHash); // Convert to string and remove outer JSON object symbols {}

          var magicComment = json5.stringify(jsonContent).slice(1, -1); // Add as a new leading comment

          importArg.addComment("leading", magicComment);
        }
      }
    }
  };
}
function shouldPrintComment(comment) {
  // Keep pure function markers which are generated by some plugins
  // See sideEffects option: https://github.com/mishoo/UglifyJS2
  if (/[#@]__PURE__/.exec(comment)) {
    return true;
  } // Keep JSON5 magic comments used for Webpack hints


  if (/^\s?webpack[A-Z][A-Za-z]+:/.exec(comment)) {
    return true;
  }

  return false;
}

exports.default = smartWebpackImport;
exports.shouldPrintComment = shouldPrintComment;
//# sourceMappingURL=index.cjs.js.map
